import styled from "styled-components";
import { gray } from "@core/colors/colors";

export type TableColumnTypes<T> = {
  label: string | JSX.Element;
  accessKey?: keyof T;
  renderCell?: ({ row, idx }: { row: T; idx: number }) => JSX.Element;
  renderValue?: ({ row, idx }: { row: T; idx: number }) => any;
  className?: string;
};

export type TableProps<T> = {
  columns: TableColumnTypes<T>[];
  tableData: T[];
  isLoading?: boolean;
};

export const Table = <T extends {}>({
  columns,
  tableData,
  isLoading = true,
}: TableProps<T>) => {
  const renderClassName = (classNames: (string | undefined)[]) => {
    return classNames.join(" ");
  };

  const renderThead = () => {
    return columns.map((column, idx) => (
      <div
        key={String(column.accessKey || idx)}
        className={renderClassName(["cell", "th-cell", column.className])}
      >
        {column.label}
      </div>
    ));
  };

  const renderTbody = () => {
    return tableData.map((row, idx) => (
      <section key={idx} className="tbody-tr">
        {columns.map((column, idx) => {
          const { accessKey, className, renderValue, renderCell } = column;

          let value = undefined;

          if (renderCell) {
            value = renderCell({ row, idx });
          } else if (renderValue) {
            value = renderValue({ row, idx });
          } else if (accessKey) {
            value = row[accessKey];
          }

          return (
            <div
              key={String(column.accessKey || idx)}
              className={renderClassName(["cell", "td-cell", className])}
            >
              {value}
            </div>
          );
        })}
      </section>
    ));
  };

  const renderSkeleton = () => {
    return Array.from({ length: 5 }, (_, idx) => (
      <div className="tbody-tr">
        {columns.map((column, columnIdx) => (
          <Skeleton key={`${String(column.accessKey || columnIdx)}_${idx}`}>
            <div className={"common-table_skeleton-item"} />
          </Skeleton>
        ))}
      </div>
    ));
  };

  return (
    <Container className="common-table">
      <TheadContainer className="common-table__thead-container">
        {renderThead()}
      </TheadContainer>
      <TbodyContainer className="common-table__tbody-container">
        {isLoading ? renderSkeleton() : renderTbody()}
      </TbodyContainer>
    </Container>
  );
};

const Container = styled.div`
  width: 100%;
  display: flex;
  flex-direction: column;
`;

const TheadContainer = styled.div`
  display: flex;
  align-items: center;

  .th-cell {
    flex: 1;
  }
`;

const TbodyContainer = styled.div`
  display: flex;
  flex-direction: column;

  .tbody-tr {
    display: flex;

    .td-cell {
      flex: 1;
    }
  }
`;

const Skeleton = styled.div`
  width: 100%;

  .common-talbe__skeleton-item {
    width: 100%;
    height: "16px";
    background-color: ${gray[200]};
    border-radius: 40px;
  }
`;
